/*
 * Open Source Hardware Logo generator
 * for Arduino Leonardo
 * & Linux g++ (see UNIT_TEST).
 *
 * by Hans Schou <chlor@schou.dk> 2014-01-20
 */
  
#include <math.h>

using namespace std;

enum statePoint_t { 
  START_KEYHOLE1, END_KEYHOLE1, HALF_TOOTH1, TOOTH_START_EDGE, TOOTH_TOP, TOOTH_END_EDGE, TOOTH_SADLE, HALF_TOOTH2, START_KEYHOLE2, START_INNER_CIRCLE, END };

enum drawingState_t { 
  FIRST_POINT, DRAW_LINE, DRAW_ARC, DRAW_ARC_SECOND_LAST, LAST_POINT };

/* atan(1) == 45° or 1/8 of a circle. 2π == 1 circle. So PI = 4*atan(1) */
#define PI (atan(1)*4)

/* Angel of tooth top and bottom is the same, but can be changed to a new variable if necessary. */
#define angelToothBottom angelToothTop

class OpenSourceHardWareLogo {
private:
  double radius;
  double angel;
  double lastX, lastY;
  statePoint_t statePoint;
  unsigned char toothCount;
  unsigned char toothNum;
  char relative;
  double topRadius;
  double saddleRadius;
  double innerRadius;
  double angelToothTop;
  double angelToothEdge;

public:
  OpenSourceHardWareLogo(int, double, char);
  drawingState_t nextPointXY(double *x, double *y);
  drawingState_t nextPointXY(int *x, int *y);
};

OpenSourceHardWareLogo::OpenSourceHardWareLogo(int newToothCount, double newTopRadius, char newRelative) {
  toothCount = newToothCount;
  topRadius = newTopRadius;
  if (relative = newRelative) {
    lastX = 0;
    lastY = 0;
  }
  saddleRadius = topRadius * 3/4;
  innerRadius = topRadius * 9/32; /* aprx ~ 1/4 */
  angelToothTop = 1.0/(toothCount*3);
  angelToothEdge = (1.0/toothCount - angelToothTop - angelToothBottom) / 2;
  statePoint = START_KEYHOLE1;
}

drawingState_t OpenSourceHardWareLogo::nextPointXY(double *x, double *y) {
  drawingState_t drawingState;
  switch (statePoint) {
  case START_KEYHOLE1:
    /* We are starting at the center of the logo.
     			 * The first move should be without pen down. */
    drawingState = FIRST_POINT;
    radius = innerRadius;
    angel = 1.0/4 + 1.0/(toothCount*2);
    statePoint = END_KEYHOLE1;
    break;
  case END_KEYHOLE1:
    /* Draw key hole radial from insideand out , same angel. */
    /* Draw radial line of key hole. From inner circle to tooth sadle, same angel. */
    drawingState = DRAW_LINE;
    radius = saddleRadius;
    statePoint = HALF_TOOTH1;
    break;
  case HALF_TOOTH1:
    /* Draw a half bootom tooth. Same saddleRadius as before. */
    drawingState = DRAW_ARC;
    angel += angelToothEdge;
    /* Set number of teeth to draw. One tooth is missing in the gear (by design). */
    toothNum = toothCount - 1;
    statePoint = TOOTH_START_EDGE;
    break;
  case TOOTH_START_EDGE:
    /* Draw to start edge of tooth. */
    drawingState = DRAW_LINE;
    radius = topRadius;
    angel += angelToothEdge;
    /* A tooth now drawn. */
    --toothNum;
    statePoint = TOOTH_TOP;
    break;
  case TOOTH_TOP:
    /* Draw the top of the tooth. topRadius is the same. */
    drawingState = DRAW_ARC;
    angel += angelToothTop;
    statePoint = TOOTH_END_EDGE;
    break;
  case TOOTH_END_EDGE:
    /* Draw end edge of tooth. From out topRadius to saddleRaius. */
    drawingState = DRAW_LINE;
    radius = saddleRadius;
    angel += angelToothEdge;
    /* If last tooth, then half tooth saddle. */
    statePoint = toothNum ? TOOTH_SADLE : HALF_TOOTH2;
    break;
  case TOOTH_SADLE:
    /* Draw the bottom saddle between two tooths. */
    drawingState = DRAW_ARC;
    radius = saddleRadius;
    angel += angelToothBottom;
    statePoint = TOOTH_START_EDGE;
    break;
  case HALF_TOOTH2:
    /* Draw ending half tooth saddle. Same saddleRadius. */
    drawingState = DRAW_ARC;
    angel += angelToothBottom / 2;
    statePoint = START_KEYHOLE2;
    break;
  case START_KEYHOLE2:
    /* Draw towards center, same angel. */
    drawingState = DRAW_LINE;
    radius = innerRadius;
    toothNum = (toothCount-1)*3;
    statePoint = START_INNER_CIRCLE;
    break;
  case START_INNER_CIRCLE:
    /* Draw inner circle. topRadius is the same. Drawing is opposite direction (counter clock wise). */
    angel -= 1.0/(toothCount*3);
    --toothNum;
    /* When reach last point stop. */
    if (toothNum) {
      drawingState = DRAW_ARC;
    } else {
      drawingState = DRAW_ARC_SECOND_LAST;
      statePoint = END;
    }
    break;
  case END:
    /* Done. Go back to center. */
    drawingState = LAST_POINT;
    radius = 0.0;
    angel = 0.0;
    statePoint = START_KEYHOLE1;
    break;
  }

  /*
   * Now this fraction coordinate system is going to be converted to 
   * polar coordinate system. This is done by angel*PI*2.
   * Then the polar coordinate system is converted to cartesian
   * coordinate system by x=radius*cos(angel) and sin() for y.
   */

  double temp;
  temp = radius * cos(angel * PI * 2);
  *x = temp;
  if (relative) {
    *x -= lastX;
    lastX = temp;
  }

  temp = radius * sin(angel * PI * 2);
  *y = temp;
  if (relative) {
    *y -= lastY;
    lastY = temp;
  }

  return drawingState;
}

drawingState_t OpenSourceHardWareLogo::nextPointXY(int *x, int *y) {
	double xd, yd;
	drawingState_t ds = nextPointXY( &xd, &yd );
	*x = (int)round(xd);
	*y = (int)round(yd);
	return ds;
}

#ifdef UNIT_TEST
/*
 * To test in a Linux environment run:
 *   g++ -DUNIT_TEST=1 -lm -o cplusplus cplusplus.cpp && ./cplusplus > ino.svg
 */

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>

int main(int argc, char *argv[]) {
  OpenSourceHardWareLogo oshw(8, 300.0, 1);
  drawingState_t drawLoop;
  double x, y;
  int svg = 1;

  if (svg) {
    printf("<svg xmlns=\"http://www.w3.org/2000/svg\"><path transform=\"translate(300,300)\" id=\"gear\" fill=\"#405fa8\" stroke=\"#18284f\" stroke-width=\"8\"\nd=\"");
  }

  while (LAST_POINT != drawLoop) {
    switch (drawLoop = oshw.nextPointXY(&x, &y)) {
    case FIRST_POINT:
      printf(svg ? "m %.2f,%.2f " : "Move to [%.2f,%.2f] and then pen down\n", x, y);
      break;
    case DRAW_LINE:
      printf(svg ? "l %.2f,%.2f " : "Line to [%.2f,%.2f]\n", x, y);
      break;
    case DRAW_ARC:
      printf(svg ? "l %.2f,%.2f " : "Arc to [%.2f,%.2f]\n", x, y);
      break;
    case DRAW_ARC_SECOND_LAST:
      printf(svg ? "  " : "Arc to [%.2f,%.2f]\n", x, y);
      break;
    case LAST_POINT:
      printf(svg ? "z\" />\n</svg>\n" : "Pen up and move to [%.2f,%.2f]\n", x, y);
      break;
    }
  }

  return EXIT_SUCCESS;
}
#else

OpenSourceHardWareLogo oshw(8, 160.0, 1);
int x, y;

void setup() {
  TX_RX_LED_INIT;
  RXLED1;
  delay(3000);
  RXLED0;
  // initialize mouse control:
  Mouse.begin();
}

int idx = 0;

void loop() {
  switch (oshw.nextPointXY(&x, &y)) {
  case FIRST_POINT:
  case DRAW_LINE:
  case DRAW_ARC:
  case DRAW_ARC_SECOND_LAST:
    if (1 == idx) {
      /* A delay is needed before the pen goes down. */
      Mouse.press(MOUSE_LEFT);
    }
    Mouse.move(x, y, 0);
    break;
  case LAST_POINT:
    /* Drawing done. Move to center. */
    delay(100);
    Mouse.release(MOUSE_LEFT); 
    delay(100);
    Mouse.move(x, y, 0);
    /* Signal done */
    TXLED0;
    RXLED1;
    /* End-less loop. */
    while (1) {
    }
    break;
  }

  ++idx;

  if (idx % 2) {
    TXLED0;
  } 
  else {
    TXLED1;
  }
  if (idx % 3) {
    RXLED0;
  } 
  else {
    RXLED1;
  }

  // a delay so the mouse doesn't move too fast:
  delay(250);
}

#endif

